import os
import sys
import json
import hashlib
import traceback
import math
import time
import random
import logging
from PIL import Image, ImageOps, ImageSequence
from PIL.PngImagePlugin import PngInfo
import numpy as np
import safetensors.torch
from io import BytesIO
import struct
import comfy.utils
from comfy.cli_args import args
import folder_paths
import latent_preview
import node_helpers

import requests
from datetime import datetime


#You can use this node to save full size images through the websocket, the
#images will be sent in exactly the same format as the image previews: as
#binary images on the websocket with a 8 byte header indicating the type
#of binary message (first 4 bytes) and the image format (next 4 bytes).

#Note that no metadata will be put in the images saved with this node.

class SaveImageGadzoinks:
    def __init__(self):
        self.output_dir = folder_paths.get_output_directory()
        self.type = "output"
        self.prefix_append = ""
        self.compress_level = 4

    @classmethod
    def INPUT_TYPES(s):
        return { "required": {"handle": ("STRING",{}),"authkey": ("STRING",{}), "age": ("INT",{"default": 17, "min": 4, "max": 17}),"private": ("INT",{"default": 0, "min": 0, "max": 1, "step": 1}), "images" : ("IMAGE", {}) },  "hidden": {"prompt": "PROMPT", "extra_pnginfo": "EXTRA_PNGINFO"}, }        
#return {"required":  {"handle": ("STRING", ),"images": ("IMAGE", ),} }

    RETURN_TYPES = ()
    FUNCTION = "save_images_gadzoinks"

    OUTPUT_NODE = True

    CATEGORY = "spoon"

    def save_images_gadzoinks(self, handle,authkey, age, private, images, prompt=None, extra_pnginfo=None):
        print(f"testing")
        filename_prefix = ""
        # support save metadata for latent sharing
        prompt_info = ""
        if prompt is not None:
            prompt_info = json.dumps(prompt)
        metadata = None
        if not args.disable_metadata:
            metadata = {"prompt": prompt_info}
            if extra_pnginfo is not None:
                for x in extra_pnginfo:
                    metadata[x] = json.dumps(extra_pnginfo[x])
        full_output_folder, filename, counter, subfolder, filename_prefix = folder_paths.get_save_image_path(filename_prefix, self.output_dir, images[0].shape[1], images[0].shape[0])
        filename = f".gz_{filename}"
        results = list()
        for (batch_number, image) in enumerate(images):
            i = 255. * image.cpu().numpy()
            img = Image.fromarray(np.clip(i, 0, 255).astype(np.uint8))
            metadata = None
            if not args.disable_metadata:
                metadata = PngInfo()
                if prompt is not None:
                    metadata.add_text("prompt", json.dumps(prompt))
                if extra_pnginfo is not None:
                    for x in extra_pnginfo:
                        print(f"extra_pnginfo: {json.dumps(extra_pnginfo[x])} ")
                        metadata.add_text(x, json.dumps(extra_pnginfo[x]))
            filename_with_batch_num = filename.replace("%batch_num%", str(batch_number))
            file = f"{filename_with_batch_num}_{counter:05}_.png"
            img.save(os.path.join(full_output_folder, file), pnginfo=metadata, compress_level=self.compress_level)
            results.append({
                "filename": file,
                "subfolder": subfolder,
                "type": self.type
            })
            fage = 17
            if age > 4 and age <=12 :
                fage = 12
            if age == 4:
                fage = 4
            vis = private
            extra = {}
            if prompt is not None:
                extra = json.dumps(prompt)
            form_data = {
                "handle": handle,
                "authkey": authkey,
                "keywords": "",
                "caption": "",
                "prompt": "",
                "nprompt": "",
                "maturityRating": str(fage),
                "ownerRanking": "0",
                "fileName": file,
                "fileType": "image/jpeg",
                "visibility" : vis,
                "app" : "comfyui",
                "extra" : extra
            }
            jd = json.dumps(form_data)
            url = "https://kcd4tcn593.execute-api.us-east-1.amazonaws.com/ProdV1//webupload"
            resp = requests.post(url, data=jd)
            print(f"response:{resp}")
            j = resp.json()
            print(f"resp.json:{j}")
            status = j.get("status",500)
            if status == 200:
                url = j["url"]
                print(f"url {url}")
                fields = j["fields"]
                print( f"fields {fields}" )
                #files = {'file': os.path.join(full_output_folder, file) }
                #files = {'file': open( (full_output_folder, file, 'image/jpeg')}
                #files = {'file': img. }
                files = {'file':open(os.path.join(full_output_folder, file),"rb") }
                print(f"files:{files}")
                http_response = requests.post(url, data=fields , files=files )
                print(f"http_response:{http_response}")
            elif status == 403:
                msg = j.get("message","unknown error uploading file")
                print(f"Gadzoinks Extension upload failed.  {msg}")
            os.remove(os.path.join(full_output_folder, file))
            counter += 1
        return { "ui": { "images": results } }

    def IS_CHANGED(s, images):
        return time.time()

NODE_CLASS_MAPPINGS = {
    "SaveImageGadzoinks": SaveImageGadzoinks,
}
